# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['volttron', 'volttron.driver.interfaces.modbus']

package_data = \
{'': ['*']}

install_requires = \
['pymodbus>=2.5.3,<3.0.0', 'volttron-lib-base-driver>=0.2.0rc0,<0.3.0']

setup_kwargs = {
    'name': 'volttron-lib-modbus-driver',
    'version': '0.1.1a1',
    'description': 'Modbus driver supported and maintained by the Volttron team.',
    'long_description': '# volttron-lib-modbus-driver\n\n![Passing?](https://github.com/VOLTTRON/volttron-lib-modbus-driver/actions/workflows/run-tests.yml/badge.svg)\n[![pypi version](https://img.shields.io/pypi/v/volttron-lib-modbus-driver.svg)](https://pypi.org/project/volttron-lib-modbus-driver/)\n\nVOLTTRON’s modbus driver supports the Modbus over TCP/IP protocol only.\n\n# Prerequisites\n\n* Python 3.8\n\n# Installation\n\n1. Create and activate a virtual environment.\n\n    ```shell\n    python -m venv env\n    source env/bin/activate\n    ```\n\n1. Install volttron and start the platform.\n\n    ```shell\n    pip install volttron\n\n    # Start platform with output going to volttron.log\n    volttron -vv -l volttron.log &\n    ```\n\n1. Install the volttron platform driver:\n\n    ```shell\n    vctl install volttron-platform-driver --vip-identity platform.driver --start\n    ```\n\n1. Install the volttron-lib-modbus-driver library.\n\n    ```shell\n    pip install volttron-lib-modbus-driver\n    ```\n\n1. Install the driver onto the Platform Driver.\n\n    Installing a driver in the Platform Driver Agent requires adding copies of the device configuration and registry configuration files to the Platform Driver’s configuration store.\n\n    Create a config directory and navigate to it:\n\n    ```shell\n    mkdir config\n    cd config\n    ```\n\n    Navigate to the config directory and create a driver configuration file called `modbus.config`. There are three arguments for the driver_config section of the device configuration file:\n\n    * device_address:  IP Address of the device.\n\n    * port: Port the device is listening on. Defaults to 502 which is the standard port for Modbus devices.\n\n    * slave_id:  Slave ID of the device. Defaults to 0. Use 0 for no slave.\n\n    This repo provides an example configuration in the file "modbus_example.config".\n\n    Here is an example device configuration file:\n\n    ```json\n    {\n        "driver_config": {"device_address": "10.0.0.4"},\n        "driver_type": "modbus",\n        "registry_config":"config://catalyst371.csv",\n        "interval": 120,\n        "timezone": "UTC",\n        "campus": "campus",\n        "building": "building",\n        "unit": "modbus1",\n        "heart_beat_point": "ESMMode"\n    }\n    ```\n\n    Create another file called `mobus.csv`. This CSV file will be your modbus registry configuration file. Each row configures a point on the device.\n\n    The following columns are required for each row:\n\n    * Volttron Point Name - The name by which the platform and agents running on the platform will refer to this point. For instance, if the Volttron Point Name is HeatCall1 (and using the example device configuration above) then an agent would use pnnl/isb2/hvac1/HeatCall1 to refer to the point when using the RPC interface of the actuator agent.\n\n    * Units - Used for meta data when creating point information on the historian.\n\n    * Modbus Register - A string representing how to interpret the data register and how to read it from the device. The string takes two forms:\n\n        * “BOOL” for coils and discrete inputs.\n\n        * A format string for the Python struct module. See the Python3 Struct docs for full documentation. The supplied format string must only represent one value. See the documentation of your device to determine how to interpret the registers. Some Examples:\n\n        * “>f” - A big endian 32-bit floating point number.\n\n        * “<H” - A little endian 16-bit unsigned integer.\n\n        * “>l” - A big endian 32-bit integer.\n\n    * Writable - Either TRUE or FALSE. Determines if the point can be written to. Only points labeled TRUE can be written to through the ActuatorAgent.\n\n    * Point Address - Modbus address of the point. Cannot include any offset value, it must be the exact value of the address.\n\n    * Mixed Endian - (Optional) Either TRUE or FALSE. For mixed endian values. This will reverse the order of the Modbus registers that make up this point before parsing the value or writing it out to the device. Has no effect on bit values.\n\n    The following column is optional:\n\n    * Default Value - The default value for the point. When the point is reverted by an agent it will change back to this value. If this value is missing it will revert to the last known value not set by an agent.\n\n    Any additional columns will be ignored. It is common practice to include a Point Name or Reference Point Name to include the device documentation’s name for the point and Notes and Unit Details for additional information about a point.\n\n    The following is an example of a Modbus registry configuration file:\n\n    ```csv\n    Reference Point Name,Volttron Point Name,Units,Units Details,Modbus Register,Writable,Point Address,Default Value,Notes\n    CO2Sensor,ReturnAirCO2,PPM,0.00-2000.00,>f,FALSE,1001,,CO2 Reading 0.00-2000.0 ppm\n    CO2Stpt,ReturnAirCO2Stpt,PPM,1000.00 (default),>f,TRUE,1011,1000,Setpoint to enable demand control ventilation\n    Cool1Spd,CoolSupplyFanSpeed1,%,0.00 to 100.00 (75 default),>f,TRUE,1005,75,Fan speed on cool 1 call\n    Cool2Spd,CoolSupplyFanSpeed2,%,0.00 to 100.00 (90 default),>f,TRUE,1007,90,Fan speed on Cool2 Call\n    Damper,DamperSignal,%,0.00 - 100.00,>f,FALSE,1023,,Output to the economizer damper\n    DaTemp,DischargeAirTemperature,F,(-)39.99 to 248.00,>f,FALSE,1009,,Discharge air reading\n    ESMEconMin,ESMDamperMinPosition,%,0.00 to 100.00 (5 default),>f,TRUE,1013,5,Minimum damper position during the energy savings mode\n    FanPower,SupplyFanPower, kW,0.00 to 100.00,>f,FALSE,1015,,Fan power from drive\n    FanSpeed,SupplyFanSpeed,%,0.00 to 100.00,>f,FALSE,1003,,Fan speed from drive\n    HeatCall1,HeatCall1,On / Off,on/off,BOOL,FALSE,1113,,Status indicator of heating stage 1 need\n    HeartBeat,heartbeat,On / Off,on/off,BOOL,FALSE,1114,,Status indicator of heating stage 2 need\n    ```\n\n    Add modbus.csv and modbus.config to the configuration store:\n\n    ```\n    vctl config store platform.driver devices/campus/building/modbus modbus.config\n    vctl config store platform.driver modbus.csv modbus.csv --csv\n    ```\n\n1. Observe Data\n\n    To see data being published to the bus, install a [Listener Agent](https://pypi.org/project/volttron-listener/):\n\n    ```\n    vctl install volttron-listener --start\n    ```\n\n    Once installed, you should see the data being published by viewing the Volttron logs file that was created in step 2.\n    To watch the logs, open a separate terminal and run the following command:\n\n    ```\n    tail -f <path to folder containing volttron.log>/volttron.log\n    ```\n\n# Development\n\nPlease see the following for contributing guidelines [contributing](https://github.com/eclipse-volttron/volttron-core/blob/develop/CONTRIBUTING.md).\n\nPlease see the following helpful guide about [developing modular VOLTTRON agents](https://github.com/eclipse-volttron/volttron-core/blob/develop/DEVELOPING_ON_MODULAR.md)\n\n\n# Disclaimer Notice\n\nThis material was prepared as an account of work sponsored by an agency of the\nUnited States Government.  Neither the United States Government nor the United\nStates Department of Energy, nor Battelle, nor any of their employees, nor any\njurisdiction or organization that has cooperated in the development of these\nmaterials, makes any warranty, express or implied, or assumes any legal\nliability or responsibility for the accuracy, completeness, or usefulness or any\ninformation, apparatus, product, software, or process disclosed, or represents\nthat its use would not infringe privately owned rights.\n\nReference herein to any specific commercial product, process, or service by\ntrade name, trademark, manufacturer, or otherwise does not necessarily\nconstitute or imply its endorsement, recommendation, or favoring by the United\nStates Government or any agency thereof, or Battelle Memorial Institute. The\nviews and opinions of authors expressed herein do not necessarily state or\nreflect those of the United States Government or any agency thereof.\n',
    'author': 'Mark Bonicillo',
    'author_email': 'volttron@pnnl.gov',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/VOLTTRON/volttron-lib-modbus-driver',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
